CARGO_TIMESERIES_RESOURCE = "/time-series"

CARGO_MOVEMENTS_RESOURCE = "/cargo-movements/search"
VESSEL_MOVEMENTS_RESOURCE = "/vessel-movements/search"
EIA_FORECASTS_RESOURCE = "/search/eia-forecasts"

ATTRIBUTES_REFERENCE = "/reference/attributes"
GEOGRAPHIES_REFERENCE = "/reference/geographies"
VESSELS_REFERENCE = "/reference/vessels"
PRODUCTS_REFERENCE = "/reference/products"
CORPORATIONS_REFERENCE = "/reference/charterers"

API_FQDN = "https://api.vortexa.com"
API_URL = f"{API_FQDN}/v5"
